/* $Id: portio.c,v 1.2 1996/03/26 22:52:18 james Exp james $ */
/*
 * $Log: portio.c,v $
 * Revision 1.2  1996/03/26 22:52:18  james
 * Put the function return types on the line before the function's name.
 *
 * Revision 1.1  1995/03/30 20:24:53  james
 * Initial revision
 *
 */

#include <stdio.h>		/* sprintf() */
#include <stdlib.h>		/* free() */
#include <fcntl.h>		/* open(), close() */
#include <unistd.h>		
#include <string.h>		/* strdup() */
#include <assert.h>

#define ASSERT(x) assert(x)

#include "jy_sg.h"

/*
 * Module data
 */
static int port_fd=-1;		/* fd for the port IO device. */
static char *port_dev_name = NULL;


static const char *default_port_device = "/dev/port";


#define FATAL_PORT_IO_ERROR(name) \
do { \
  extern char *port_dev_name; \
  char buf[100]; \
  sprintf(buf, #name "() on %s", port_dev_name); \
  perror(buf); \
  exit(1); \
} while(0)			/* NOTE: NO SEMICOLON HERE */


int
open_port_device(const char *name)
{
  extern int port_fd;

  if (NULL == name)
    name = default_port_device;
  
  if (port_fd != -1)
    close(port_fd);
  
  port_fd = open(name, O_RDWR);
  if (port_fd == -1)
    {
      return 0;
    }
  else
    {
      if (port_dev_name)
	free(port_dev_name);
      port_dev_name = strdup(name);
      
      return 1;
    }
}

enum SgStatus
close_port_device(void)
{
  extern int port_fd;
  extern char *port_dev_name;

  if (port_fd != -1)
    {
      if (close(port_fd) == -1)
	{
	  return SG_OK;		/* failure */
	}
      port_fd = -1;
    }
  if (port_dev_name)
    free(port_dev_name);
  
  port_dev_name = NULL;
  
  return SG_ERROR;
}

static void
io_seek(IOADDR addr)
{
  extern int port_fd;

  ASSERT(port_fd != -1);
  
  if (lseek(port_fd, (off_t)addr, SEEK_SET) == -1)
    {
      FATAL_PORT_IO_ERROR(lseek);
    }
}

      
BYTE
read_io_byte(IOADDR addr)
{
  extern int port_fd;
  BYTE ret;

  ASSERT(port_fd != -1);
  ASSERT(sizeof(BYTE) == 1);	/* sizeof() gives size in machine bytes */
  
  io_seek(addr);
  ret = 0;
  if (read(port_fd, &ret, sizeof(ret)) == -1)
    {
      FATAL_PORT_IO_ERROR(read);
    }
  return ret;
}

void
write_io_byte(IOADDR addr, BYTE val)
{
  extern int port_fd;

  ASSERT(port_fd != -1);
  ASSERT(sizeof(BYTE) == 1);	/* sizeof() gives size in machine bytes */
  
  io_seek(addr);
  if (write(port_fd, &val, sizeof(val)) == -1)
    {
      FATAL_PORT_IO_ERROR(write);
    }
}
